<?php

namespace App\Exports;

use App\Models\Consumable;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithMapping;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Style\Alignment;
use PhpOffice\PhpSpreadsheet\Style\Border;
use PhpOffice\PhpSpreadsheet\Style\Fill;
use DateTime;

class ConsumablesExport implements FromCollection, WithHeadings, WithMapping, WithEvents
{
    protected $year;
    protected $month;

    public function __construct($year, $month)
    {
        $this->year = $year;
        $this->month = $month;
    }

    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        return Consumable::whereYear('date', $this->year)
            ->whereMonth('date', $this->month)
            ->get();
    }

    /**
     * @return array
     */
    public function headings(): array
    {
        return [
            'ID',
            'Name',
            'Quantity',
            'Date',
            'Amount Spent',
            'Notes',
        ];
    }

    /**
     * @param $consumable
     * @return array
     */
    public function map($consumable): array
    {
        return [
            $consumable->id,
            $consumable->name,
            $consumable->quantity,
            $consumable->date->format('Y-m-d'),
            $consumable->amount_spent,
            $consumable->notes,
        ];
    }

    /**
     * @return array
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet->getDelegate();

                // Set title, month, and year at the top
                $sheet->setCellValue('A1', 'Consumables Report');
                $sheet->setCellValue('A2', 'Month: ' . DateTime::createFromFormat('!m', $this->month)->format('F'));
                $sheet->setCellValue('A3', 'Year: ' . $this->year);

                // Merge cells for the title
                $sheet->mergeCells('A1:F1');
                $sheet->mergeCells('A2:F2');
                $sheet->mergeCells('A3:F3');

                // Apply styling for the title
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 16,
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                    ],
                ]);

                // Apply styling for the month and year
                $sheet->getStyle('A2:A3')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                    ],
                ]);

                // Write the header row manually to avoid duplication
                $headerRow = 5; // Assuming headers are in row 5
                $sheet->fromArray([$this->headings()], NULL, 'A' . $headerRow);

                // Apply styling for the header row
                $sheet->getStyle('A' . $headerRow . ':F' . $headerRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                        ],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'startColor' => [
                            'rgb' => 'D9EAD3', // Light green color
                        ],
                    ],
                ]);

                // Add data starting from the row after headers
                $dataRow = $headerRow + 1;
                $collection = $this->collection();
                $sheet->fromArray($collection->map([$this, 'map'])->toArray(), NULL, 'A' . $dataRow);

                // Add total row
                $lastRow = $sheet->getHighestRow();
                $totalRow = $lastRow + 1;
                $sheet->setCellValue('D' . $totalRow, 'Total');
                $sheet->setCellValue('E' . $totalRow, '=SUM(E' . $dataRow . ':E' . $lastRow . ')'); // Total calculation

                // Apply styling for the total row
                $sheet->getStyle('A' . $totalRow . ':F' . $totalRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                        ],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'startColor' => [
                            'rgb' => 'F4CCCC', // Light red color
                        ],
                    ],
                ]);
            },
        ];
    }
}
