<?php

namespace App\Exports;

use App\Models\Unit;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithStyles;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;

class MonthlyRevenueExport implements FromCollection, WithHeadings, ShouldAutoSize, WithStyles
{
    protected $month;
    protected $year;

    public function __construct($month, $year)
    {
        $this->month = $month;
        $this->year = $year;
    }

    public function collection()
    {
        $monthNumber = date('n', strtotime($this->month)); // Convert month name to numeric format

        $units = Unit::with(['leases.payments' => function ($query) {
            $query->whereYear('payment_date', $this->year);
        }])->get();

        $data = [];
        foreach ($units as $unit) {
            $row = [
                $unit->property->property_name,
                $unit->unit_name,
            ];

            foreach (range(1, 12) as $monthIndex) {
                $payment = $unit->leases
                    ->flatMap(function ($lease) use ($monthIndex) {
                        return $lease->payments->filter(function ($payment) use ($monthIndex) {
                            return $payment->payment_date->format('n') == $monthIndex &&
                                $payment->payment_date->format('Y') == $this->year;
                        });
                    })
                    ->sum('Amount');

                $row[] = $payment ?: 0; // Add payment amount or 0 if no payment
            }

            $data[] = $row; // Ensure $row is added to data
        }

        return collect($data);
    }

    public function headings(): array
    {
        return [
            ['Monthly Revenue Report for ' . $this->year],
            ['Property Name', 'Unit #', 'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC'],
        ];
    }

    public function styles(Worksheet $sheet)
    {
        // Apply styles to the year row
        $sheet->getStyle('A1:N1')->applyFromArray([
            'font' => ['bold' => true],
            'alignment' => ['horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER],
        ]);
        // Set row height for the year row
        $sheet->getRowDimension(1)->setRowHeight(20);

        // Apply styles to the headings row
        $sheet->getStyle('A2:N2')->applyFromArray([
            'font' => ['bold' => true],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => ['rgb' => 'FFFF00']
            ],
            'borders' => [
                'allBorders' => [
                    'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                    'color' => ['argb' => 'FF000000']
                ],
            ],
        ]);
        // Set row height for the headings row
        $sheet->getRowDimension(2)->setRowHeight(20);

        // Optional: Set row height for data rows (if needed)
        foreach (range(3, $sheet->getHighestRow()) as $rowIndex) {
            $sheet->getRowDimension($rowIndex)->setRowHeight(18);
        }

        // Set column widths
        $sheet->getColumnDimension('A')->setWidth(20); // Property Name
        $sheet->getColumnDimension('B')->setWidth(15); // Unit #
        foreach (range('C', 'N') as $columnID) { // JAN to DEC
            $sheet->getColumnDimension($columnID)->setWidth(15);
        }

        return [];
    }
}
