<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Property;
use Illuminate\Support\Facades\DB;
use App\Models\Unit;
use App\Models\Client;
use Yajra\DataTables\Facades\DataTables;

class PropertyController extends Controller
{
    public function index(Request $request)
    {
        $property = Property::orderBy('created_at', 'desc')->get();

        if ($request->ajax()) {
            // Query to fetch properties with unit counts using a separate query
            $properties = Property::select([
                'id',
                'property_name',
                'street_address',
                'city',
                'state',
                'zip'
            ])
                ->withCount('units');

            return DataTables::of($properties)
                ->addColumn('full_address', function ($property) {
                    return $property->street_address . ', ' . $property->city . ', ' . $property->state . ', ' . $property->zip;
                })
                ->addColumn('unit_count', function ($property) {
                    return $property->units_count ?? 0; // Ensure default value if count is null
                })
                ->addColumn('view_units', function ($property) {
                    return '<a href="' . url('/admin/unit', $property->id) . '" class="px-2 py-1 font-bold text-white bg-blue-500 rounded hover:bg-blue-700">View Units</a>';
                })
                ->addColumn('actions', function ($property) {
                    return '<button onclick="openModal(\'editItemModal' . $property->id . '\')" class="px-2 py-1 font-bold text-white bg-blue-500 rounded hover:bg-blue-700">Edit</button>';
                })
                ->rawColumns(['view_units', 'actions'])
                ->toJson();
        }

        return view('CMS.Property.property', compact('property'));
    }
    public function allunits(Request $request)
    {
        if ($request->ajax()) {
            $units = Unit::leftJoin('lease', function ($join) {
                $join->on('units.unit_id', '=', 'lease.unit_id')
                    ->where('lease.status', '=', 'active');
            })
                ->leftJoin('properties', 'properties.id', '=', 'units.property_id')
                ->leftJoin('clients', 'clients.client_id', '=', 'lease.client_id')
                ->select(
                    'units.*',
                    'clients.client_id as c_id',
                    'lease.status as lease_status', // Alias the lease status
                    'properties.*', // Alias the property name
                    'clients.*' // Alias the client name
                )
                ->orderBy('units.created_at', 'asc')
                ->get();

            return DataTables::of($units)
                ->addColumn('billing', function ($unit) {
                    return '<a href="' . url('/lease', $unit->unit_id) . '"  class="px-4 py-2 mb-2 font-bold text-white bg-blue-500 rounded hover:bg-blue-700">View Details</a>';
                })
                ->addColumn('size', function ($unit) {
                    return $unit->size . ' sqm'; // Assuming 'size' is a field in the 'units' table
                })
                ->addColumn('actions', function ($unit) {
                    return '<button onclick="openModal(\'editUnitModal' . $unit->unit_id . '\')" class="px-2 py-1 font-bold text-white bg-blue-500 rounded hover:bg-blue-700">Edit</button>
                            <button onclick="openModal(\'deleteModal' . $unit->unit_id . '\')" class="px-2 py-1 font-bold text-white bg-red-500 rounded hover:bg-red-700">Delete</button>';
                })
                ->rawColumns(['billing', 'actions'])
                ->toJson();
        }

        // Fetch units with default ordering when not in AJAX request
        $units = Unit::orderBy('created_at', 'desc')->get();
        return view('CMS.Property.allunit', compact('units'));
    }








    public function viewunits($id)
    {
        $property = Property::findOrFail($id);
        $clients = Client::all();
        $properties = Property::all();

        if (request()->ajax()) {
            $units = Unit::where('property_id', $id)->get();

            return DataTables::of($units)
                ->addColumn('view_lease', function ($unit) {
                    return '<a href="' . url('/lease', $unit->unit_id) . '" class="px-2 py-1 font-bold text-white bg-blue-500 rounded hover:bg-blue-700">View Lease</a>';
                })
                ->addColumn('size', function ($unit) {
                    return $unit->size . ' sqm'; // Assuming 'size' is a field in the 'units' table
                })
                ->addColumn('actions', function ($unit) {
                    return '<button onclick="openModal(\'editUnitModal' . $unit->unit_id . '\')" class="px-2 py-1 font-bold text-white bg-blue-500 rounded hover:bg-blue-700">Edit</button>
                            <button onclick="openModal(\'deleteModal' . $unit->unit_id . '\')" class="px-2 py-1 font-bold text-white bg-red-500 rounded hover:bg-red-700">Delete</button>';
                })
                ->rawColumns(['view_lease', 'actions'])
                ->toJson();
        }

        $units = Unit::where('property_id', $id)->get();
        return view('CMS.Property.units', compact('property', 'units', 'clients', 'properties'));
    }



    public function create()
    {
        return view('properties.create');
    }


    public function store(Request $request)
    {
        // Validate the request
        $request->validate([
            'property_name' => 'required|string|max:255',
            'street_address' => 'required|string|max:255',
            'city' => 'required|string|max:255',
            'state' => 'required|string|max:255',
            'zip' => 'required|string|max:10',
            'latitude' => 'nullable|numeric',
            'longitude' => 'nullable|numeric',
        ]);

        try {
            // Create the property
            $propertyData = [
                'property_name' => $request->input('property_name'),
                'street_address' => $request->input('street_address'),
                'city' => $request->input('city'),
                'state' => $request->input('state'),
                'zip' => $request->input('zip'),
                'latitude' => $request->input('latitude'),
                'longitude' => $request->input('longitude'),
            ];

            $property = Property::create($propertyData);

            return redirect()->back()->with('success', 'Property created successfully.');
        } catch (\Exception $e) {
            // Handle any exceptions, such as database errors
            return redirect()->back()->withErrors(['error' => 'Failed to create property. Please try again.'])->withInput();
        }
    }


    public function update(Request $request, $id)
    {
        $request->validate([
            'property_name' => 'required|string|max:255',
            'street_address' => 'required|string|max:255',
            'city' => 'required|string|max:255',
            'state' => 'required|string|max:255',
            'zip' => 'required|string|max:10',
            // Add validation rules for units fields if applicable
        ]);

        $property = Property::findOrFail($id);

        $property->property_name = $request->input('property_name');
        $property->street_address = $request->input('street_address');
        $property->city = $request->input('city');
        $property->state = $request->input('state');
        $property->zip = $request->input('zip');

        // Update any other fields here as needed

        $property->save();

        return redirect()->back()->with('success', 'Property updated successfully.');
    }
    public function updateunits(Request $request)
    {
        $id = $request->input('id');


        $unit = Unit::findOrFail($id);

        $unit->unit_name = $request->input('unit_name');
        $unit->size = $request->input('size');
        $unit->rate_type = $request->input('rate_type');
        $unit->status = $request->input('status');
        $unit->rate = $request->input('rate');
        $unit->yearly_escalation = $request->input('yearly_escalation');
        $unit->status = $request->input('status');

        $unit->save();

        return redirect()->back()->with('success', 'Unit updated successfully.');
    }



    public function delete($id)
    {
        try {
            $property = Property::findOrFail($id);
            $property->delete();

            return redirect()->back()->with('success', 'Property deleted successfully.');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Cannot delete property. There are related records in other tables.');
        }
    }
    public function deleteunit($id)
    {
        try {

            DB::table('units')->where('unit_id', $id)->delete();

            return redirect()->back()->with('success', 'Unit deleted successfully.');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Cannot delete Unit. There are related records in other tables.');
        }
    }

    public function storenewunit(Request $request)
    {
        // Retrieve the existing property using the provided property_id
        $property = Property::findOrFail($request->input('property_id'));

        // Check if the request contains units to be added
        if ($request->filled('units')) {
            // Iterate over each unit's data and create a new unit associated with the property
            foreach ($request->input('units') as $unitData) {
                $property->units()->create([
                    'unit_name' => $unitData['unit_name'],
                    'size' => $unitData['size'],
                    'rate_type' => $unitData['rate_type'],
                    'rate' => $unitData['rate'],
                    'yearly_escalation' => $unitData['yearly_escalation'],
                    'status' => 'Vacant',
                ]);
            }
        }

        // Redirect back with a success message
        return redirect()->back()->with('success', 'Units have been successfully added to the property.');
    }
}
